# -*- coding: utf-8 -*-
import os
import sys
import inspect
from functools import wraps
from flask import Flask
from flask_ssm.springframework.boot import SpringApplication
from flask_ssm.utils.context_utils import has_app_context
from flask_ssm.utils.module_utils import get_package_from_path, find_member_from_multi_level_package
from flask_ssm.utils.time_utils import execute_time


class Test:
    def __init__(self, func):
        wraps(func)(self)
        self.func = func

    def __call__(self, *args, **kwargs):
        if has_app_context():
            return self.func(*args, **kwargs)
        else:
            # change directory
            _path_tree_ = get_package_from_path(os.path.dirname(sys.argv[0]), False, True)
            # find package name
            sps = find_member_from_multi_level_package(lambda x: isinstance(x, SpringApplication), _path_tree_)
            # build app context
            app = Flask(".".join(_path_tree_))
            for sp in sps:
                sp[1].init_app(app)
                setattr(inspect.getmodule(self.func), "__orm__", sp[1].orm)
            # run original function
            with app.app_context():
                result, use_time = execute_time(self.func, *args, **kwargs)
                app.logger.info("执行用时: %dms" % use_time)
            return result



# -*- coding: utf-8 -*-
from types import FunctionType
from functools import wraps
from flask import Flask
import flask.globals as globs


def add_app_context(app: Flask, func: FunctionType) -> FunctionType:
    """
    给函数加上上下文\n
    :param app: Flask对象
    :param func: 函数
    :return: 有上下文的函数
    """
    @wraps(func)
    def result(*args, **kwargs):
        with app.app_context():
            return func(*args, **kwargs)
    return result


def has_app_context() -> bool:
    """
    判断是否有上下文\n
    :return: 是否有上下文
    """
    _app_ctx_stack = getattr(globs, "_app_ctx_stack", None)
    return _app_ctx_stack.top is not None



# -*- coding: utf-8 -*-
from typing import Type
from functools import wraps
import inspect
from flask import current_app


class Transactional:
    """
    事务管理器\n
    """
    def __init__(self, rollback_for: Type = Exception):
        """
        构造方法\n
        :param rollback_for: 捕获的异常
        """
        self.rollback_for = rollback_for

    def __call__(self, func):
        """
        执行函数\n
        :param func: 原函数
        :return:
        """
        if not issubclass(self.rollback_for, Exception):
            raise TypeError("%s is not subclass of Exception" % str(self.rollback_for))

        @wraps(func)
        def wrapper(*args, **kwargs):
            _module_ = inspect.getmodule(func)
            db = getattr(_module_, "__orm__")
            result = None
            try:
                result = func(*args, **kwargs)
            except self.rollback_for as e:
                db.session.rollback()
                current_app.logger.exception(e)
            else:
                db.session.commit()
            finally:
                return result
        return wrapper



# -*- coding: utf-8 -*-
import os
import sys
from typing import List, Optional, Union, Tuple, Any
from types import ModuleType, FunctionType, LambdaType
import inspect
from flask import Blueprint, Flask


def walk_sub_modules(package: ModuleType) -> List[ModuleType]:
    """
    递归获取子模块\n
    :param package: 包
    :return: 子模块
    """
    for _path_, _folders_, _files_ in os.walk(os.path.dirname(package.__file__), topdown=True):
        for _folder_ in _folders_:
            if _folder_.startswith("__"):
                _folders_.remove(_folder_)
        _package_name_ = package.__name__ + _path_.replace(os.path.dirname(package.__file__), "").replace(os.sep, ".")
        _files_ = list(filter(lambda x: (not x.startswith("__")) and x.lower().endswith(".py"), _files_))
        _files_ = list(map(lambda x: x[0:-3], _files_))
        for _file_ in _files_:
            _module_ = __import__(_package_name_ + "." + _file_, fromlist=[_file_])
            yield _module_


def try_to_import(*module_names: str) -> Optional[ModuleType]:
    """
    根据包名导入模块\n
    :param module_names: 包名
    :return: 模块
    """
    for module_name in module_names:
        try:
            result = __import__(module_name, fromlist=module_name.split(".")[-1])
            return result
        except ModuleNotFoundError as e:
            pass
    return None


def get_package_from_path(_path_: str, return_name: bool, change_dir: bool) -> Union[str, List[str]]:
    """
    从目录获取包名\n
    :param _path_: 目录
    :param return_name: 是 返回 package.name.service；否 返回 ['package', 'name', 'service']
    :param change_dir: 是否改变路径
    :return: 包名
    """
    _path_tree_ = list()
    while os.path.exists(os.path.join(_path_, "__init__.py")):
        _path_tree_.append(os.path.basename(_path_))
        _path_ = os.path.dirname(_path_)
        if change_dir:
            os.chdir(_path_)
    _path_tree_ = _path_tree_[::-1]
    if return_name:
        return ".".join(_path_tree_)
    else:
        return _path_tree_


def find_member_from_multi_level_package(func: Union[FunctionType, LambdaType], _path_tree_: List[str]) -> List[Tuple[str, Any]]:
    """
    从多级包中寻找成员。从父向子查找，找到第一组就退出\n
    :param func: 筛选函数
    :param _path_tree_: 多级包名字组成的列表，如['package', 'name', 'service']。该参数在遍历过程中被修改，最终变为找到的位置
    :return: 寻找到的成员
    """
    while True:
        _package_name_ = ".".join(_path_tree_)
        module = __import__(_package_name_, fromlist=[_package_name_.split(".")[-1]])
        sps = inspect.getmembers(module, func)
        if len(sps) > 0:
            return sps
        if len(_path_tree_) > 0:
            _path_tree_.pop()
        else:
            return list()


def blueprint_from_module(func: FunctionType) -> Blueprint:
    """
    从函数所在的模块中提取Blueprint类对象\n
    :param func: 函数
    :return: Blueprint类的对象
    """
    _module_ = inspect.getmodule(func)
    _bps_ = inspect.getmembers(_module_, lambda x: isinstance(x, Blueprint))
    if _bps_:
        result = _bps_[-1][1]
    else:
        result = Blueprint(
            _module_.__name__.replace(".", "_"),
            _module_.__name__,
            static_url_path="",
            root_path=os.path.dirname(os.path.abspath(sys.argv[0]))
        )
        setattr(_module_, "__blueprint__", result)
    return result


def run_with_outside_server(self: Flask, server_name: Optional[str] = None) -> None:
    """
    使用外部web服务器运行时的必要设置\n
    :param self: Flask对象
    :param server_name: 服务器软件名称，暂未使用
    """
    for _, bp in self.blueprints.items():
        bp.root_path = os.getcwd()



# -*- coding: utf-8 -*-
import re
import json
from functools import wraps
from typing import Union, Collection, Type
import inspect
from urllib.parse import unquote
from flask import request, Response
from flask_ssm.utils.module_utils import blueprint_from_module


class RequestMethod:
    """
    请求方法\n
    """
    GET = "GET"
    HEAD = "HEAD"
    POST = "POST"
    PUT = "PUT"
    PATCH = "PATCH"
    DELETE = "DELETE"
    OPTIONS = "OPTIONS"
    TRACE = "TRACE"


class RequestMapping:
    """
    用它修饰请求接口函数\n
    """
    def __init__(self, value: str, method: Union[str, Collection[str]]):
        """
        构造方法\n
        :param value: 请求路径
        :param method: 请求方法
        """
        self.rule = re.sub(r'\{([^}]+)\}', r'<\1>', value)
        if type(method) is str:
            self.methods = [method]
        else:
            self.methods = list(method)

    def __call__(self, func):
        """
        执行函数\n
        :param func: 原函数
        :return:
        """
        @wraps(func)
        def result(*args, **kwargs):
            if request.is_json:
                json_value = request.get_json(silent=True)
                if json_value is not None:
                    kwargs.update(json_value)
            elif request.mimetype.startswith("application/x-www-form-urlencoded") or request.mimetype.startswith("multipart/form-data"):
                values = request.values
                values = dict(zip(values.keys(), map(lambda x: unquote(x), values.values())))
                kwargs.update(values)
                if request.mimetype.startswith("multipart/form-data"):
                    kwargs.update(request.files)
            kwargs.update(dict(zip(inspect.signature(func).parameters.keys(), args)))
            if "" in kwargs.keys():
                kwargs.pop("")
            _module_ = inspect.getmodule(func)
            _inner_result_ = func(**kwargs)
            if inspect.getmembers(_module_, lambda x: x is RestController):
                return Response(status=200, response=json.dumps(_inner_result_, default=lambda x: x.__dict__), content_type="application/json;charset=utf-8")
            else:
                return _inner_result_
        bp = blueprint_from_module(func)
        return bp.route(self.rule, methods=self.methods)(result)


class GetMapping(RequestMapping):
    """
    用它修饰GET请求接口函数\n
    """
    def __init__(self, value: str):
        """
        构造方法\n
        :param value: 请求路径
        """
        super().__init__(value, RequestMethod.GET)

    def __call__(self, func):
        """
        执行函数\n
        :param func: 原函数
        :return:
        """
        return super().__call__(func)


class PostMapping(RequestMapping):
    """
    用它修饰POST请求接口函数\n
    """
    def __init__(self, value: str):
        """
        构造方法\n
        :param value: 请求路径
        """
        super().__init__(value, RequestMethod.POST)

    def __call__(self, func):
        """
        执行函数\n
        :param func: 原函数
        :return:
        """
        return super().__call__(func)


class PutMapping(RequestMapping):
    """
    用它修饰PUT请求接口函数\n
    """
    def __init__(self, value: str):
        super().__init__(value, RequestMethod.PUT)

    def __call__(self, func):
        """
        执行函数\n
        :param func: 原函数
        :return:
        """
        return super().__call__(func)


class DeleteMapping(RequestMapping):
    """
    用它修饰DELETE请求接口函数\n
    """
    def __init__(self, value: str):
        super().__init__(value, RequestMethod.DELETE)

    def __call__(self, func):
        """
        执行函数\n
        :param func: 原函数
        :return:
        """
        return super().__call__(func)


class PatchMapping(RequestMapping):
    """
    用它修饰PATCH请求接口函数\n
    """
    def __init__(self, value: str):
        super().__init__(value, RequestMethod.PATCH)

    def __call__(self, func):
        """
        执行函数\n
        :param func: 原函数
        :return:
        """
        return super().__call__(func)


class ExceptionHandler:
    """
    用来统一处理同一个模块的接口异常\n
    """
    def __init__(self, value: Type[Exception] = Exception):
        """
        构造方法\n
        :param value: 捕获的异常
        """
        self.exception_cls = value

    def __call__(self, func):
        """
        执行函数\n
        :param func: 原函数
        :return:
        """
        bp = blueprint_from_module(func)
        return bp.errorhandler(self.exception_cls)(func)


class ResponseBody:
    """
    把函数返回值的json写入响应体\n
    """
    def __init__(self, func):
        """
        构造函数\n
        :param func: 装饰的函数
        """
        wraps(func)(self)
        self.func = func

    def __call__(self, *args, **kwargs):
        """
        调用函数\n
        :param args: 变长参数
        :param kwargs: 关键字参数
        :return: 修改后的返回值
        """
        result = self.func(*args, **kwargs)
        if type(result) is Response:
            return result
        else:
            return Response(status=200, response=json.dumps(result, default=lambda x: x.__dict__), content_type="application/json;charset=utf-8")


class RestController:
    pass



# -*- coding: utf-8 -*-
from logging.config import dictConfig
import os


__path__ = os.path.join(os.getcwd(), "logs")
if not os.path.exists(__path__):
    os.mkdir(__path__)


dictConfig({
    "version": 1,
    "root": {"level": "DEBUG", "handlers": ["console", "debug", "info", "warn", "error"]},
    "formatters": {"default": {"format": "%(asctime)s - %(levelname)s - %(message)s"}},
    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
            "stream": "ext://flask.logging.wsgi_errors_stream",
            "formatter": "default"
        },
        "debug": {
            "class": "logging.handlers.TimedRotatingFileHandler",
            "level": "DEBUG",
            "encoding": "utf-8",
            "filename": os.path.join(__path__, "DEBUG"),
            "when": "MIDNIGHT",
            "backupCount": 1,
            "formatter": "default"
        },
        "info": {
            "class": "logging.handlers.TimedRotatingFileHandler",
            "level": "INFO",
            "encoding": "utf-8",
            "filename": os.path.join(__path__, "INFO"),
            "when": "MIDNIGHT",
            "backupCount": 1,
            "formatter": "default"
        },
        "warn": {
            "class": "logging.handlers.TimedRotatingFileHandler",
            "level": "WARN",
            "encoding": "utf-8",
            "filename": os.path.join(__path__, "WARN"),
            "when": "MIDNIGHT",
            "backupCount": 1,
            "formatter": "default"
        },
        "error": {
            "class": "logging.handlers.TimedRotatingFileHandler",
            "level": "ERROR",
            "encoding": "utf-8",
            "filename": os.path.join(__path__, "ERROR"),
            "when": "MIDNIGHT",
            "backupCount": 1,
            "formatter": "default"
        }
    }
})



# -*- coding: utf-8 -*-
from flask_sqlalchemy import SQLAlchemy


class Controller:
    """
    接口\n
    """
    pass


class Service:
    """
    业务逻辑\n
    """
    pass


class Repository:
    """
    数据交互\n
    """
    db = SQLAlchemy()






# -*- coding: utf-8 -*-
import os
import re
import sys
import typing
from typing import Type, List, Tuple, Dict, Union, NoReturn, Optional
from functools import wraps
import pkgutil
from inspect import signature
import inspect
from collections.abc import Generator
from flask import current_app
from sqlalchemy import text
from sqlalchemy.engine.cursor import CursorResult
from sqlalchemy.engine.result import MappingResult
from sqlalchemy.engine.row import Row
from flask_sqlalchemy import SQLAlchemy
from flask_ssm.springframework.stereotype import Repository
from flask_ssm.utils.module_utils import try_to_import
from flask_ssm.utils.type_utils import __get_origin__, pojo_private_properties, validate_single_value


if sys.version_info >= (3, 9):
    from types import GenericAlias
else:
    GenericAlias = getattr(typing, "_GenericAlias")
if sys.version_info >= (3, 8):
    from typing import get_args
else:
    from typing_inspect import get_args


class Mapper:
    """
    ORM映射\n
    """
    def __init__(self, result_type: Optional[Union[Type, GenericAlias]] = CursorResult, namespace: Optional[str] = None, *args, **kwargs):
        """
        构造方法\n
        :param result_type: 返回类型
        :param namespace: 多个数据源时指定的连接数据源，单个数据源指定为None
        :param args: 传递给result_type的变长参数
        :param kwargs: 传递给result_type的关键字参数
        """
        self.result_type = result_type
        self.namespace = namespace
        self.args = args
        self.kwargs = kwargs

    def __call__(self, func):
        """
        执行函数\n
        :param func: 原函数
        :return:
        """
        @wraps(func)
        def wrapper(*params, **kwparams):
            _module_ = inspect.getmodule(func)
            db: SQLAlchemy = getattr(_module_, "__orm__")
            sql: str = func(*params, **kwparams)
            sql = re.sub(r'#\{(\w+)\}', r':\1', sql)
            if not isinstance(sql, str):
                raise TypeError("error in @Mapper, return result of mapper function must be a sql string.")
            kwparams.update(dict(zip(signature(func).parameters.keys(), params)))
            sql = re.sub(r'\${(\w+)}', lambda x: str(kwparams.pop(x.group(1))), sql)
            # try to import modules:
            pd = try_to_import("pandas")
            np = try_to_import("numpy")
            try:
                # process all result_types
                if self.result_type is CursorResult:                                                 # CursorResult
                    result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                    return result
                elif self.result_type is MappingResult:                                              # MappingResult
                    result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                    return result.mappings()
                elif pd is not None and self.result_type is pd.DataFrame:                            # pd.DataFrame
                    result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                    return pd.DataFrame(result.fetchall(), columns=result.mappings().keys(), *self.args, **self.kwargs).replace([None], np.nan)
                elif pd is not None and self.result_type is pd.Series:                               # pd.Series
                    result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                    values = list(zip(*result.fetchall()))
                    keys = list(result.mappings().keys())
                    if len(keys) > 1:
                        current_app.logger.warning("found %d columns, only pick columns[0]: %s" % (len(keys), keys[0]))
                    if len(values) > 0:
                        return pd.Series(values[0], name=keys[0], *self.args, **self.kwargs).replace([None], np.nan)
                    else:
                        return pd.Series(values, name=keys[0], *self.args, **self.kwargs).replace([None], np.nan)
                elif np is not None and self.result_type is np.ndarray:                              # np.ndarray
                    result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                    sql_result = np.array(result.fetchall(), *self.args, **self.kwargs)
                    sql_result = np.where(sql_result == None, np.nan, sql_result)
                    return sql_result
                elif self.result_type in (None, NoReturn):                                           # NoReturn or None
                    db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                    return None
                elif self.result_type is Row:                                                        # Row
                    result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                    return result.fetchone()
                elif __get_origin__(self.result_type) is list:
                    if self.result_type in (List, list):                                             # List or list
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        fetch_result = result.fetchone()
                        return None if fetch_result is None else list(fetch_result)
                    _class_ = get_args(self.result_type)[0]
                    if __get_origin__(_class_) is dict:                                         # List[Dict] or List[dict]
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        return result.mappings().all()
                    elif __get_origin__(_class_) is tuple:                                      # List[Tuple] or List[tuple]
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        return list(map(tuple, result))
                    elif __get_origin__(_class_) is list:                                       # List[List] or List[list]
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        return list(map(list, result))
                    elif __get_origin__(_class_) is Row:                                        # List[Row]
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        return result.fetchall()
                    else:
                        if issubclass(_class_, db.Model):                                       # List[Pojo]
                            return db.session.query(_class_).from_statement(text(sql)).params(**kwparams).all()
                        else:                                                                   # List[T]
                            result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                            keys = list(result.mappings().keys())
                            if len(keys) > 1:
                                current_app.logger.warning("found %d columns, only pick columns[0]: %s" % (len(keys), keys[0]))
                            _res_ = [validate_single_value(_class_, x[0]) for x in result]
                            return _res_
                elif __get_origin__(self.result_type) is tuple:
                    if self.result_type in (Tuple, tuple):                                          # Tuple or tuple
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        fetch_result = result.fetchone()
                        return None if fetch_result is None else tuple(fetch_result)
                    _class_ = get_args(self.result_type)[0]
                    if __get_origin__(_class_) is dict:                                         # Tuple[Dict] or Tuple[dict]
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        return tuple(result.mappings().all())
                    elif __get_origin__(_class_) is tuple:                                      # Tuple[Tuple] or Tuple[tuple]
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        return tuple(map(tuple, result))
                    elif __get_origin__(_class_) is list:                                       # Tuple[List] or Tuple[list]
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        return tuple(map(list, result))
                    elif __get_origin__(_class_) is Row:                                        # Tuple[Row]
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        return tuple(result.fetchall())
                    else:
                        if issubclass(_class_, db.Model):                                       # Tuple[Pojo]
                            return tuple(db.session.query(_class_).from_statement(text(sql)).params(**kwparams).all())
                        else:                                                                   # Tuple[T]
                            result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                            keys = list(result.mappings().keys())
                            if len(keys) > 1:
                                current_app.logger.warning("found %d columns, only pick columns[0]: %s" % (len(keys), keys[0]))
                            _res_ = tuple(validate_single_value(_class_, x[0]) for x in result)
                            return _res_
                elif issubclass(__get_origin__(self.result_type), Generator):
                    if self.result_type in (Generator, typing.Generator):                            # Generator
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        keys = list(result.mappings().keys())
                        if len(keys) > 1:
                            return (x for x in result)
                        else:
                            return (x[0] for x in result)
                    _class_ = get_args(self.result_type)[0]
                    if __get_origin__(_class_) is dict:                                         # Generator[Dict, None, None] or Generator[dict, None, None]
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        return (x for x in result.mappings())
                    elif __get_origin__(_class_) is tuple:                                      # Generator[Tuple, None, None] or Generator[tuple, None, None]
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        return (x for x in result)
                    elif __get_origin__(_class_) is list:                                       # Generator[List, None, None] or Generator[list, None, None]
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        return (list(x) for x in result)
                    else:
                        if issubclass(_class_, db.Model):                                       # Generator[Pojo, None, None]
                            return (x for x in db.session.query(_class_).from_statement(text(sql)).params(**kwparams))
                        else:                                                                   # Generator[T, None, None]
                            result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                            keys = list(result.mappings().keys())
                            if len(keys) > 1:
                                current_app.logger.warning("found %d columns, only pick columns[0]: %s" % (len(keys), keys[0]))
                            return (validate_single_value(_class_, x[0]) for x in result)
                elif __get_origin__(self.result_type) is dict:
                    if self.result_type is Dict or self.result_type is dict:                              # Dict or dict
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        return result.mappings().fetchone()
                    _class_ = get_args(self.result_type)[1]
                    if __get_origin__(_class_) is list:                                         # Dict[str, List] or Dict[str, list]
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        values = list(map(list, list(zip(*result.fetchall()))))
                        keys = list(result.mappings().keys())
                        return dict(zip(keys, values)) if len(values) > 0 else {key: [] for key in keys}
                    elif __get_origin__(_class_) is tuple:                                      # Dict[str, Tuple] or Dict[str, tuple]
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        values = list(zip(*result.fetchall()))
                        keys = list(result.mappings().keys())
                        return dict(zip(keys, values)) if len(values) > 0 else {key: tuple() for key in keys}
                    else:                                                                       # Dict[str, Any]
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        values = result.fetchone()
                        keys = list(result.mappings().keys())
                        return dict(zip(keys, values)) if values else {key: None for key in keys}
                else:
                    _class_ = self.result_type
                    if issubclass(_class_, db.Model):                                           # Pojo
                        return db.session.query(_class_).from_statement(text(sql)).params(**kwparams).first()
                    else:                                                                       # T
                        result: CursorResult = db.session.execute(sql, kwparams, bind_arguments={"bind": db.engines[self.namespace]})
                        keys = list(result.mappings().keys())
                        if len(keys) > 1:
                            current_app.logger.warning("found %d fields, only pick fields[0]: %s" % (len(keys), keys[0]))
                        fetch_result = result.fetchone()
                        _res_ = None if fetch_result is None else fetch_result[0]
                        return validate_single_value(_class_, _res_)
            finally:
                if self.result_type in (CursorResult, MappingResult):
                    current_app.logger.warning(f"result_type={self.result_type}, 未关闭session, 请手动关闭session!")
                else:
                    db.session.close()
        return wrapper


class TableName:
    """
    一个Pojo类\n
    """
    def __init__(self, value: Optional[str] = None, schema: Optional[str] = None):
        """
        构造方法\n
        :param value: 表名
        :param schema: schema名称
        """
        self.table_name = value
        self.schema = schema

    def __call__(self, cls):
        """
        让cls继承db.Model，并且注入__tablename__\n
        :param cls: 装饰的类
        :return:
        """
        package_tree = inspect.getmodule(cls).__name__.split(".")[0:-2]
        sub_trees = inspect.getmodule(cls).__file__.split(os.sep)[0:-2]
        while len(package_tree) > 1:
            base_package = ".".join(package_tree)
            sub_packages = pkgutil.iter_modules([os.sep.join(sub_trees)])
            for sub_package in sub_packages:
                if sub_package.ispkg:
                    sub_name = base_package + "." + sub_package.name
                    sub_module = __import__(sub_name, fromlist=[sub_package.name])
                    repos = inspect.getmembers(sub_module, lambda x: x is Repository)
                    if repos:
                        db = repos[0][1].db
                        metadata = pojo_private_properties(cls)
                        if self.table_name:
                            metadata["__tablename__"] = self.table_name
                        if self.schema:
                            if "__table_args__" in metadata.keys():
                                __table_args__ = metadata["__table_args__"]
                                for i, __table_arg__ in enumerate(__table_args__):
                                    if type(__table_arg__) is dict:
                                        metadata["__table_args__"][i]["schema"] = self.schema
                                        break
                            else:
                                metadata["__table_args__"] = ({"schema": self.schema}, )
                        if self.table_name and not issubclass(cls, db.Model):
                            cls = type(cls.__name__, (db.Model, cls), metadata)
                            return cls
            package_tree.pop()
            sub_trees.pop()
        return cls



# -*- coding: utf-8 -*-
import setuptools
import flask_ssm


base_url = "https://github.com/ZongXR/Flask-SSM"
with open("./README.md", "r", encoding="utf-8") as f:
    long_description = f.read()
    long_description = long_description.replace("./test/demo", base_url + "/tree/main/test/demo")
    long_description = long_description.replace("./app.py", base_url + "/tree/main/app.py")
packages = list(filter(lambda x: not x.startswith("test"), setuptools.find_packages()))
requires_list = open('./requirements.txt', 'r', encoding='utf8').readlines()
requires_list = [x.strip() for x in requires_list if (not x.startswith("PyMySQL")) and (not x.startswith("setuptools"))]


setuptools.setup(
    name="Flask-SSM",
    version=flask_ssm.__version__,
    author="Xiangrui Zong",
    author_email="zxr@tju.edu.cn",
    description="A Flask based package imitate with Spring Framework",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=base_url,
    packages=packages,
    license="GNU General Public License v3 (GPLv3)",
    classifiers=[
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Natural Language :: Chinese (Simplified)',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries',
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    ],
    python_requires='>=3.7,<3.11',
    install_requires=requires_list
)



# -*- coding: utf-8 -*-
import inspect
from functools import wraps
from pydantic import ValidationError
from flask_ssm.utils.type_utils import validate_params, validate_value_with_type


class Validated:
    def __init__(self, func):
        wraps(func)(self)
        self.func = func

    def __call__(self, *args, **kwargs):
        kwargs.update(dict(zip(inspect.signature(self.func).parameters.keys(), args)))
        kwargs, errors = validate_params(self.func, kwargs)
        if errors:
            raise TypeError(f"参数类型校验出错: {str(errors)}")
        result = self.func(**kwargs)
        try:
            if "return" in self.func.__annotations__.keys():
                return validate_value_with_type(self.func.__annotations__["return"], result)
            else:
                return result
        except ValidationError as e:
            raise TypeError(f"返回值类型校验出错: {str(e.errors()[0])}")
